@@ -1,6 +1,12 @@
 from tml.utils import Stack
 from tml import uml
 from django.utils.html import conditional_escape
+from collections import namedtuple
+
+
+Mark = namedtuple('Mark', ['tagname', 'classname', 'href'])
+
+
 
 class ParseError(Exception):
     def __init__(self, lineno, message, data=''):
@@ -88,79 +94,76 @@ class InlineCloseMark():
 InlineCloseMarkOrLineEndClosure = InlineCloseMark()
     
 
-class MarkInfo():
-    # immutable mark info
-    def __init__(self, tagname, closeType, control):
-        self.tagname = tagname 
-        self.closeType = closeType
-        self.control = control 
-        
-        
         
 class MarkData():
     '''
-    control
-        whatever text mark triggers creating this class
     mark
-        what will be written as tag
+        a parsed representation of a mark
+    control
+        the text mark which triggered creating this class
     '''
-    def __init__(self, tagname, closeType, control, open_lineno=None):
-        self.tagname = tagname 
+    def __init__(self, mark, closeType, control, open_lineno=None):
+        self.mark = mark 
         self.closeType = closeType
         self.control = control 
         self.open_lineno = open_lineno
-
-    @classmethod
-    def from_generic(class_object, info, open_lineno=None):
-        '''Call as
-           d = MarkData.from_generic(info, lineno)
-        '''
-        return class_object(
-            info.tagname, 
-            info.closeType, 
-            info.control, 
-            open_lineno=open_lineno
-        )
         
     def open(self, b):
-        b.append('<{}>'.format(self.tagname))
+        if (self.mark.classname):
+            b.append('<{} class="{}">'.format(self.mark.tagname, self.mark.classname))
+        else:
+            b.append('<{}>'.format(self.mark.tagname))
 
     def close(self, b):
-        b.append('</{}>'.format(self.tagname))
+        b.append('</{}>'.format(self.mark.tagname))
         
     def error_str(self):
         return "control '{}' opened at line:{}".format(self.control, self.open_lineno)
 
     def __repr__(self):
-        return "MarkData(tagname='{}', closeType={}, control='{}', open_lineno={})".format(
-            self.tagname,
+        return "MarkData(mark='{}', closeType={}, control='{}', open_lineno={})".format(
+            self.mark,
             self.closeType,
             self.control, 
             self.open_lineno,
         )        
         
+#?        
 class ParagraphMarkData(MarkData):
     def __init__(self, **kwargs):
-        super().__init__('p', InlineStartOrBlockMarkClosure, '\\n', **kwargs)
-
+        super().__init__(Mark('p', '', ''), InlineStartOrBlockMarkClosure, '\\n', **kwargs)
 
 
 
 class AnchorMarkData(MarkData):
-    def __init__(self, href, open_lineno=None):
-        self.href = href
-        super().__init__('a', TargetedMarkClosure, '[', open_lineno)
-                
+    # This hold now?
+    def __init__(self, mark, open_lineno=None):
+        super().__init__(mark, TargetedMarkClosure, '{', open_lineno)
+    
+    # tagname is on mark?
     def open(self, b):
-        b.append('<a href="{}">'.format(self.href))
-
+        href =  '#'   
+        if (self.mark.href):
+            href = self.mark.href
+        if (self.mark.classname):
+            b.append('<a href="{}" class="{}">'.format(href, self.mark.classname))
+        else:
+            b.append('<a href="{}">'.format(href))
 
 
+#? figure something better, surely
 class PreMarkData(MarkData):
-    def __init__(self, tagname, open_lineno=None):
-        super().__init__('pre', TargetedMarkClosure, '?', open_lineno)
+    def __init__(self, mark, open_lineno=None):
+        super().__init__(mark, TargetedMarkClosure, '?', open_lineno)
 
-  
+    def open(self, b):
+        if (self.mark.classname):
+            b.append('<pre class="{}">'.format(self.mark.classname))
+        else:
+            b.append('<pre>')
+            
+    def close(self, b):
+        b.append('</pre>')  
 
 
 
@@ -183,29 +186,26 @@ class Parser:
     BlockBracketMarks = ['#', '+', '>']
     BlockListElementMarks = ['-', '~', ':']
     BlockTitleMark = '='
-    #BlockSingleMarks = ['*']
+    InlineOpenMark = '{'
+    InlineCloseMark = '}'
     
     BlockSignificantMarks = ( 
         BlockBracketMarks
-        #+ BlockSingleMarks
         )
 
+    # overridable attribute for markdata class
     pre_markdata = PreMarkData
 
-    shortcutMarkInfo = {
-      '#' : MarkInfo('div', TargetedMarkClosure, '#'), 
-      '+' : MarkInfo('ul', TargetedMarkClosure, '+'), 
-      '-' : MarkInfo('li', ListElementOrCloseClosure, '-'),
-      '~' : MarkInfo('dt', ListElementOrCloseClosure, '~'),
-      ':' : MarkInfo('dd', ListElementOrCloseClosure, ':'),
-      #'?' : MarkInfo('pre', TargetedMarkClosure, '?'),
-      '>' : MarkInfo('blockquote', TargetedMarkClosure, '>'),
-      #'*' : MarkInfo('img', '*'),
-      '[' : MarkInfo('span', InlineCloseMarkOrLineEndClosure, '[')
+    shortcutBlockTags = {
+          '#' : 'div', 
+          '+' : 'ul', 
+          '?' : 'pre',
+          '>' : 'blockquote',
     }
-   
-    #NB Not rendered through stack, so needs no MArkData
-    anonymousShortcutTagnameInfo = { 
+
+    #NB Not rendered through stack, or with attributes, so needs no 
+    # MarkData
+    listElementTags = { 
       '-': 'li',
       '~': 'dt',
       ':': 'dd',
@@ -213,6 +213,7 @@ class Parser:
         
     #! uml should go aftretr TML, probably?
     def __init__(self, uml=False):
+        self.inlineB = []
         self.reset(uml=uml)
 
     def reset(self, uml=False):
@@ -296,24 +297,34 @@ class Parser:
     def parseInlineControl(self):
         '''
         Eat input from current pos until whitespace.
-        Note this is a tricky function. It eats the limiting 
+        Note this is a tricky function. It eats a limiting 
         whitespace. It also will not error, returning as much as it can
-        find, and quietly stopping at lineends.
+        find. It stops quietly at lineends.
         '''
-        tagnameB = []
+        tagclassnameB = []
         hrefB = []
         c = self.cpGet()
         while (c and not self.isWhitespace(c) and c != '('):
-            tagnameB.append(c)
+            tagclassnameB.append(c)
             c = self.cpGet()
+            
+        # now get tag and class names
+        tagclassname =  ''.join(tagclassnameB) 
+        splitC = tagclassname.split('.', 1)
+        tagname = splitC[0]
+        classname = ''
+        if (len(splitC) > 1):
+            classname = splitC[1] 
+ 
+        # Now parse hrefs
         if (c == '('):
             c = self.cpGet()
             while (c and (c != ')')):
                 hrefB.append(c)            
                 c = self.cpGet()
             #self.cpGet()  
-            self.cpSkip()          
-        return (''.join(tagnameB), ''.join(hrefB))
+            self.cpSkip()  
+        return Mark(tagname, classname, ''.join(hrefB))
 
 
     # Block stack handling
@@ -369,48 +380,61 @@ class Parser:
         
     def onInlineOpen(self, b):
         # positioned on the char after the control
-        tagname, href = self.parseInlineControl()     
-        if (not tagname):
-            # shortcut data available
-            info = self.shortcutMarkInfo['[']
-            d = MarkData.from_generic(info, self.lineno)
-        elif (tagname == 'a'):
-            if (not href):
-                href = '#'
-            d = AnchorMarkData(href, self.lineno)
+        mark = self.parseInlineControl()     
+        if (mark.tagname == 'a'):
+            d = AnchorMarkData(mark, self.lineno)
         else:
+            if (not mark.tagname):
+                mark = Mark('span', mark.classname, '')
             # assume generic closure
-            d = MarkData(tagname, TargetedMarkClosure, '[', self.lineno)
+            d = MarkData(mark, TargetedMarkClosure, self.InlineOpenMark, self.lineno)
         self.markOpenPush(b, d)
 
     def onInlineClose(self, b):
-        self.expectedHeadPopClose(b, '[')
+        self.expectedHeadPopClose(b, self.InlineOpenMark)
 
-    def processInlineContent(self, b):
+    def inlineBuilderWrite(self, b):
+        l = ''.join(self.inlineB)
+        if (self.uml):
+            l = uml.all(l)
+        b.append(l)
+        self.inlineB = []
+                
+    def processInlineContent(self, b):        
         # used once set up for inline material
         self.skipWhiteSpace()
-        if (self.uml):
-            # We are beyond block tags, with only inlines to
-            # consider.
-            # Slice the line, as UML may tinker with overall length
-            # and this ensures our start pos.
-            self.line = uml.all(self.line[self.linepos:])
-            self.linelen = len(self.line)
-            self.linepos = 0
+        # Cute, but need to skip inline marks too
+        # if (self.uml):
+            # # We are beyond block tags, with only inlines to
+            # # consider.
+            # # Slice the line, as UML may tinker with overall length
+            # # and this ensures our start pos.
+            # self.line = uml.all(self.line[self.linepos:])
+            # self.linelen = len(self.line)
+            # self.linepos = 0
             
-        p = self.cpGet()
-        while (p):
-            if (p == '['):
+        cp = self.cpGet()
+
+        while (cp):
+            if (cp == self.InlineOpenMark):
+                self.inlineBuilderWrite(b)
+                
+                # Process the mark
                 self.onInlineOpen(b)
             
                 # slack between a tagname and the written content. Must 
                 # be one space, but may be more
                 self.skipWhiteSpace()
-            elif (p == ']'):
+            elif (cp == self.InlineCloseMark):
+                self.inlineBuilderWrite(b)
                 self.onInlineClose(b)
             else:
-                b.append(p)
-            p = self.cpGet()
+                self.inlineB.append(cp)
+                #b.append(cp)
+            cp = self.cpGet()
+            
+        # write anything left
+        self.inlineBuilderWrite(b)
                             
     def paragraphOpenPush(self, b):
         markData = ParagraphMarkData(open_lineno=self.lineno)
@@ -447,7 +471,8 @@ class Parser:
             
             
     # block level
-
+    
+    #? like anonymouse list items
     def onHeadlineControl(self, b):
         # It's a non-list element block
         # open or close, this is the end of phrase content.
@@ -463,25 +488,24 @@ class Parser:
         self.processInlineContent(b)
         b.append('</{}>'.format(tagname))
 
+    def parse_mark_data(self, data):
+        splitC = data.split('.', 1)
+        tagname = splitC[0]
+        classname = ''
+        if (len(splitC) > 1):
+            classname = splitC[1] 
+        return Mark(tagname, classname, '')
+        
     def parseAndProcessBlockOpen(self, b, control):
         '''
         Create, render and push data to the closeStack. 
         '''
         # parse blockcontrol
         # Note this eats the delimiting space        
-        tagname = self.getUntilWhiteSpace()     
-           
-        # munch the space
-        #self.cpGet()
-        
-        # convert shortcuts like '+' to MarkInfo, else make one
-        if (tagname in self.shortcutMarkInfo):
-            # shortcut data available
-            info = self.shortcutMarkInfo[tagname]
-            d = MarkData.from_generic(info, self.lineno)
-        else:
-            # assume generic closure
-            d = MarkData(tagname, TargetedMarkClosure, control, self.lineno)
+        mark = self.parse_mark_data(self.getUntilWhiteSpace())
+        if (mark.tagname in self.shortcutBlockTags):
+            mark = Mark(self.shortcutBlockTags[mark.tagname], mark.classname, '')
+        d = MarkData(mark, TargetedMarkClosure, control, self.lineno)
         self.markOpenPush(b, d)
                 
     def onBlockControl(self, b, control):
@@ -529,16 +553,16 @@ class Parser:
             self.inEscape = True
                 
             # open mark
-            #self.parseAndProcessBlockOpen(b, control)
-            tagname = self.getUntilWhiteSpace()  
-            d = self.pre_markdata(tagname, self.lineno)
+            # Note this eats the delimiting space        
+            mark = self.parse_mark_data(self.getUntilWhiteSpace())
+            d = self.pre_markdata(mark, self.lineno)
             self.markOpenPush(b, d)
         
     def onListElementControl(self, b, control):
         # List element hard-left controls arrive here.
         # The position is after the control
         # only need one control, no parsing
-        # The contol is skipped.
+        # The control is skipped.
 
         # close all self-closing blocks, includes previous list 
         # elements and inline content
@@ -552,7 +576,8 @@ class Parser:
             
             # special block, closes on inline starts and non-list blocks 
             #! put somewhere configurable
-            d = MarkData('ul', InlineStartOrNonListBlockClosure, '+', open_lineno=self.lineno)
+            #?
+            d = MarkData(Mark('ul', '', ''), InlineStartOrNonListBlockClosure, '+', open_lineno=self.lineno)
             self.markOpenPush(b, d)
             
             # update local head var
@@ -564,14 +589,16 @@ class Parser:
 
         # test the head/surrounding block for special anonymous closure
         if (e and e.closeType == InlineStartOrNonListBlockClosure):
-            # an anonymous list
-            tagname = self.anonymousShortcutTagnameInfo[control]
+            # an anonymous list element
+            tagname = self.listElementTags[control]
             b.append('<{}><p>'.format(tagname) )
             self.processInlineContent(b)
             b.append('</p></{}>'.format(tagname))
         else:
-            info = self.shortcutMarkInfo[control]
-            d = MarkData.from_generic(info, self.lineno)
+            # handled as block-level tag
+            tagname = self.listElementTags[control]
+            d = MarkData(Mark(tagname, '', ''), ListElementOrCloseClosure, control, self.lineno)
+
             self.markOpenPush(b, d)
             self.onPostBlockInlineContent(b)
 
@@ -591,7 +618,7 @@ class Parser:
         elif (first_char in self.BlockSignificantMarks):
             self.onBlockControl(b, self.cpGet())
         elif (first_char == '='):
-           self.onHeadlineControl(b) 
+           self.onHeadlineControl(b)
         elif (first_char in self.BlockListElementMarks):
             self.onListElementControl(b, self.cpGet())
         elif (first_char == '?'):
@@ -602,10 +629,10 @@ class Parser:
             if (not self.inEscape):
                 self.onInlineStart(b)
             else:
-                # write in the line, no parsing
+                # Escaped. Write in the line, no parsing
                 b.append(conditional_escape(self.line))
                 
-                # append what would be there (but TML makes space, usually)
+                # append what would be there (but TML removes, usually)
                 b.append('\n')                 
         
     def close(self, b):
@@ -626,10 +653,11 @@ class Parser:
 
 
 class PreCodeBlockMarkData(MarkData):
-    def __init__(self, tagname, open_lineno=None):
-        super().__init__(None, TargetedMarkClosure, '?', open_lineno)
+    def __init__(self, mark, open_lineno=None):
+        super().__init__(mark, TargetedMarkClosure, '?', open_lineno)
                   
     def open(self, b):
+        #? class
         b.append('<figure><pre><code contenteditable spellcheck="false">')
 
     def close(self, b):
@@ -640,16 +668,18 @@ class CodeBlockParser(Parser):
         
         
         
+        
 class PreCodeBlockPrismMarkData(MarkData):
     '''
     Markup <pre> areas for code snippets highlighted with prism.js.
     '''
-    def __init__(self, tagname, open_lineno=None):
+    def __init__(self, mark, open_lineno=None):
         # tagname used as language indicator
-        super().__init__(tagname, TargetedMarkClosure, '?', open_lineno)
+        super().__init__(mark, TargetedMarkClosure, '?', open_lineno)
                   
     def open(self, b):
-        lang = 'none' if (self.tagname == '?') else self.tagname
+        # no classname handling
+        lang = 'none' if (self.mark.tagname == '?') else self.mark.tagname
         b.append('<figure><pre><code contenteditable spellcheck="false" class="language-{0}">'.format(lang))
 
     def close(self, b):
